@@ -4,7 +4,7 @@ from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond.backend import TableHandler
+from trytond import backend
 from trytond.const import OPERATORS
 
 __all__ = ['Template', 'Product']
@@ -47,16 +47,18 @@ class Template(ModelSQL, ModelView):
     default_uom = fields.Many2One('product.uom', 'Default UOM', required=True,
         states=STATES, depends=DEPENDS)
     default_uom_category = fields.Function(
-        fields.Many2One('product.uom.category', 'Default UOM Category',
-            on_change_with=['default_uom']),
-        'on_change_with_default_uom_category')
+        fields.Many2One('product.uom.category', 'Default UOM Category'),
+        'on_change_with_default_uom_category',
+        searcher='search_default_uom_category')
     active = fields.Boolean('Active', select=True)
     products = fields.One2Many('product.product', 'template', 'Variants',
         states=STATES, depends=DEPENDS)
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
+        sql_table = cls.__table__()
 
         super(Template, cls).__register__(module_name)
 
@@ -65,11 +67,14 @@ class Template(ModelSQL, ModelView):
         table.not_null_action('category', 'remove')
 
         # Migration from 2.2: new types
-        cursor.execute('UPDATE "' + cls._table + '" '
-            'SET consumable = %s WHERE type = %s', (True, 'consumable'))
-        cursor.execute('UPDATE "' + cls._table + '" '
-            'SET type = %s WHERE type IN (%s, %s)',
-            ('goods', 'stockable', 'consumable'))
+        cursor.execute(*sql_table.update(
+                columns=[sql_table.consumable],
+                values=[True],
+                where=sql_table.type == 'consumable'))
+        cursor.execute(*sql_table.update(
+                columns=[sql_table.type],
+                values=['goods'],
+                where=sql_table.type.in_(['stockable', 'consumable'])))
 
     @staticmethod
     def default_active():
@@ -98,10 +103,28 @@ class Template(ModelSQL, ModelView):
                 'write_uid', 'write_date'))
         return [Product.default_get(fields_names)]
 
+    @fields.depends('default_uom')
     def on_change_with_default_uom_category(self, name=None):
         if self.default_uom:
             return self.default_uom.category.id
 
+    @classmethod
+    def search_default_uom_category(cls, name, clause):
+        return [('default_uom.category',) + tuple(clause[1:])]
+
+    @classmethod
+    def create(cls, vlist):
+        vlist = [v.copy() for v in vlist]
+        for values in vlist:
+            values.setdefault('products', None)
+        return super(Template, cls).create(vlist)
+
+    @classmethod
+    def search_global(cls, text):
+        for id_, rec_name, icon in super(Template, cls).search_global(text):
+            icon = icon or 'tryton-product'
+            yield id_, rec_name, icon
+
 
 class Product(ModelSQL, ModelView):
     "Product Variant"
@@ -117,8 +140,7 @@ class Product(ModelSQL, ModelView):
     active = fields.Boolean('Active', select=True)
     default_uom = fields.Function(fields.Many2One('product.uom',
             'Default UOM'), 'get_default_uom', searcher='search_default_uom')
-    type = fields.Function(fields.Selection(TYPES, 'Type',
-            on_change_with=['template']),
+    type = fields.Function(fields.Selection(TYPES, 'Type'),
         'on_change_with_type', searcher='search_type')
     list_price_uom = fields.Function(fields.Numeric('List Price',
         digits=(16, 4)), 'get_price_uom')
@@ -126,12 +148,19 @@ class Product(ModelSQL, ModelView):
         digits=(16, 4)), 'get_price_uom')
 
     @classmethod
-    def __setup__(cls):
-        super(Product, cls).__setup__()
-        # XXX order by id until order by joined name is possible
-        # but at least products are grouped
-        cls.rec_name.order_field = ("%(table)s.code %(order)s, "
-            "%(table)s.id %(order)s")
+    def order_rec_name(cls, tables):
+        pool = Pool()
+        Template = pool.get('product.template')
+        product, _ = tables[None]
+        if 'template' not in tables:
+            template = Template.__table__()
+            tables['template'] = {
+                None: (template, product.template == template.id),
+                }
+        else:
+            template = tables['template']
+        return [product.code] + Template.name.convert_order('name',
+            tables['template'], Template)
 
     @staticmethod
     def default_active():
@@ -152,12 +181,10 @@ class Product(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        ids = map(int, cls.search([('code',) + clause[1:]], order=[]))
-        if ids:
-            ids += map(int, cls.search([('template.name',) + clause[1:]],
-                    order=[]))
-            return [('id', 'in', ids)]
-        return [('template.name',) + clause[1:]]
+        return ['OR',
+            ('code',) + tuple(clause[1:]),
+            ('template.name',) + tuple(clause[1:]),
+            ]
 
     def get_default_uom(self, name):
         return self.template.default_uom.id
@@ -166,6 +193,7 @@ class Product(ModelSQL, ModelView):
     def search_default_uom(cls, name, clause):
         return [('template.default_uom',) + tuple(clause[1:])]
 
+    @fields.depends('template')
     def on_change_with_type(self, name=None):
         if self.template:
             return self.template.type
@@ -189,6 +217,12 @@ class Product(ModelSQL, ModelView):
                 res[product.id] = getattr(product, field)
         return res
 
+    @classmethod
+    def search_global(cls, text):
+        for id_, rec_name, icon in super(Product, cls).search_global(text):
+            icon = icon or 'tryton-product'
+            yield id_, rec_name, icon
+
     @classmethod
     def search_domain(cls, domain, active_test=True):
         def convert_domain(domain):
@@ -209,7 +243,9 @@ class Product(ModelSQL, ModelView):
                     field = arg[0].split('.', 1)[0]
                     if not getattr(cls, field, None):
                         field = 'template.' + arg[0]
-                    result.append((field,) + tuple(arg[1:]))
+                        result.append((field,) + tuple(arg[1:]))
+                    else:
+                        result.append(arg)
                 elif isinstance(arg, list):
                     # sub-domain
                     result.append(convert_domain(arg))
