@@ -1,6 +1,8 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
+from sql import Table
+
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
@@ -35,11 +37,11 @@ class Uom(ModelSQL, ModelView):
     category = fields.Many2One('product.uom.category', 'UOM Category',
         required=True, ondelete='RESTRICT', states=STATES, depends=DEPENDS)
     rate = fields.Float('Rate', digits=(12, 12), required=True,
-        on_change=['rate'], states=STATES, depends=DEPENDS,
+        states=STATES, depends=DEPENDS,
         help=('The coefficient for the formula:\n'
             '1 (base unit) = coef (this unit)'))
     factor = fields.Float('Factor', digits=(12, 12), states=STATES,
-        on_change=['factor'], required=True, depends=DEPENDS,
+        required=True, depends=DEPENDS,
         help=('The coefficient for the formula:\n'
             'coef (base unit) = 1 (this unit)'))
     rounding = fields.Float('Rounding Precision', digits=(12, 12),
@@ -50,10 +52,13 @@ class Uom(ModelSQL, ModelView):
     @classmethod
     def __register__(cls, module_name):
         cursor = Transaction().cursor
+        model_data = Table('ir_model_data')
         # Migration from 1.6: corrected misspelling of ounce (was once)
-        cursor.execute("UPDATE ir_model_data "
-            "SET fs_id = REPLACE(fs_id, 'uom_once', 'uom_ounce') "
-            "WHERE fs_id = 'uom_once' AND module = 'product'")
+        cursor.execute(*model_data.update(
+                columns=[model_data.fs_id],
+                values=['uom_ounce'],
+                where=(model_data.fs_id == 'uom_once')
+                & (model_data.module == 'product')))
         super(Uom, cls).__register__(module_name)
 
     @classmethod
@@ -70,8 +75,8 @@ class Uom(ModelSQL, ModelView):
                 'change_uom_rate': ('If the UOM is still not used, you can '
                     'delete it otherwise you can deactivate it '
                     'and create a new one.'),
-                'invalid_factor_and_rate': ('Invalid Factor and Rate values in '
-                    'UOM "%s".'),
+                'invalid_factor_and_rate': (
+                    'Invalid Factor and Rate values in UOM "%s".'),
                 })
 
     @classmethod
@@ -104,6 +109,7 @@ class Uom(ModelSQL, ModelView):
     def default_digits():
         return 2
 
+    @fields.depends('factor')
     def on_change_factor(self):
         if (self.factor or 0.0) == 0.0:
             return {'rate': 0.0}
@@ -111,6 +117,7 @@ class Uom(ModelSQL, ModelView):
             'rate': round(1.0 / self.factor, self.__class__.rate.digits[1]),
             }
 
+    @fields.depends('rate')
     def on_change_rate(self):
         if (self.rate or 0.0) == 0.0:
             return {'factor': 0.0}
@@ -120,11 +127,10 @@ class Uom(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        ids = map(int, cls.search(['OR',
-                    (cls._rec_name,) + clause[1:],
-                    ('symbol',) + clause[1:],
-                    ], order=[]))
-        return [('id', 'in', ids)]
+        return ['OR',
+            (cls._rec_name,) + tuple(clause[1:]),
+            ('symbol',) + tuple(clause[1:]),
+            ]
 
     @staticmethod
     def round(number, precision=1.0):
@@ -139,7 +145,7 @@ class Uom(ModelSQL, ModelView):
     def check_factor_and_rate(self):
         "Check coherence between factor and rate"
         if self.rate == self.factor == 0.0:
-            return True
+            return
         if (self.rate != round(
                     1.0 / self.factor, self.__class__.rate.digits[1])
                 and self.factor != round(
@@ -148,20 +154,25 @@ class Uom(ModelSQL, ModelView):
                         self.rec_name,))
 
     @classmethod
-    def write(cls, uoms, values):
+    def write(cls, *args):
         if Transaction().user == 0:
-            return super(Uom, cls).write(uoms, values)
-        if 'rate' not in values and 'factor' not in values \
-                and 'category' not in values:
-            super(Uom, cls).write(uoms, values)
+            super(Uom, cls).write(*args)
             return
 
+        actions = iter(args)
+        all_uoms = []
+        for uoms, values in zip(actions, actions):
+            if 'rate' not in values and 'factor' not in values \
+                    and 'category' not in values:
+                continue
+            all_uoms += uoms
+
         old_uom = dict((uom.id, (uom.factor, uom.rate, uom.category.id))
-            for uom in uoms)
+            for uom in all_uoms)
 
-        super(Uom, cls).write(uoms, values)
+        super(Uom, cls).write(*args)
 
-        for uom in uoms:
+        for uom in all_uoms:
             if uom.factor != old_uom[uom.id][0] \
                     or uom.rate != old_uom[uom.id][1] \
                     or uom.category.id != old_uom[uom.id][2]:
